"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _helperPluginUtils = require("@babel/helper-plugin-utils");

var _helperModuleImports = require("@babel/helper-module-imports");

var _default = (0, _helperPluginUtils.declare)(function (_ref, opts) {
  var assertVertion = _ref.assertVertion,
      types = _ref.types;
  return {
    name: 'babel-plugin-number-calc-autorepair',
    visitor: {
      /**
       * +、-、*、/
       */
      BinaryExpression: function BinaryExpression(nodePath, file) {
        var node = nodePath.node; // 注释豁免

        if (isImmunity(file.file.ast.comments, node)) return;

        if (types.isBinaryExpression(node, {
          operator: '+'
        })) {
          var funcName = addImport(file, 'accAdd', nodePath, types);
          nodePath.replaceWith(types.callExpression(types.identifier(funcName), [node.left, node.right]));
        }

        if (types.isBinaryExpression(node, {
          operator: '-'
        })) {
          var _funcName = addImport(file, 'accSub', nodePath, types);

          nodePath.replaceWith(types.callExpression(types.identifier(_funcName), [node.left, node.right]));
        }

        if (types.isBinaryExpression(node, {
          operator: '*'
        })) {
          var _funcName2 = addImport(file, 'accMulti', nodePath, types);

          nodePath.replaceWith(types.callExpression(types.identifier(_funcName2), [node.left, node.right]));
        }

        if (types.isBinaryExpression(node, {
          operator: '/'
        })) {
          var _funcName3 = addImport(file, 'accDiv', nodePath, types);

          nodePath.replaceWith(types.callExpression(types.identifier(_funcName3), [node.left, node.right]));
        }

        if (types.isBinaryExpression(node, {
          operator: '**'
        })) {
          var _funcName4 = addImport(file, 'accExp', nodePath, types);

          nodePath.replaceWith(types.callExpression(types.identifier(_funcName4), [node.left, node.right]));
        }
      },

      /**
       * +=、-=、*=、/=
       */
      AssignmentExpression: function AssignmentExpression(nodePath, file) {
        var node = nodePath.node; // 注释豁免

        if (isImmunity(file.file.ast.comments, node)) return;

        if (types.isAssignmentExpression(node, {
          operator: '+='
        })) {
          var funcName = addImport(file, 'accAdd', nodePath, types);
          nodePath.replaceWith(types.assignmentExpression('=', node.left, types.callExpression(types.identifier(funcName), [node.left, node.right])));
        }

        if (types.isAssignmentExpression(node, {
          operator: '-='
        })) {
          var _funcName5 = addImport(file, 'accSub', nodePath, types);

          nodePath.replaceWith(types.assignmentExpression('=', node.left, types.callExpression(types.identifier(_funcName5), [node.left, node.right])));
        }

        if (types.isAssignmentExpression(node, {
          operator: '*='
        })) {
          var _funcName6 = addImport(file, 'accMulti', nodePath, types);

          nodePath.replaceWith(types.assignmentExpression('=', node.left, types.callExpression(types.identifier(_funcName6), [node.left, node.right])));
        }

        if (types.isAssignmentExpression(node, {
          operator: '/='
        })) {
          var _funcName7 = addImport(file, 'accDiv', nodePath, types);

          nodePath.replaceWith(types.assignmentExpression('=', node.left, types.callExpression(types.identifier(_funcName7), [node.left, node.right])));
        }

        if (types.isAssignmentExpression(node, {
          operator: '**='
        })) {
          var _funcName8 = addImport(file, 'accExp', nodePath, types);

          nodePath.replaceWith(types.assignmentExpression('=', node.left, types.callExpression(types.identifier(_funcName8), [node.left, node.right])));
        }
      }
    }
  };
});

exports["default"] = _default;

function isImmunity(allComments, node) {
  var comments = (allComments || []).filter(function (comment) {
    // 文件级别
    if (comment.value.replace(/^\*/, '').trim() === 'babel-disable-autofix-file') return true; // Expession 级别

    if (comment.start < node.start || comment.end > node.end) return false;
    if (comment.value.replace(/^\*/, '').trim() === 'babel-disable-autofix') return true; // 不豁免

    return false;
  });
  if (comments.length > 0) return true;
  return false;
}

function addImport(file, key, nodePath, types) {
  var list = file.file.ast.program.body || [];
  var item = list.filter(function (item) {
    return item.type === 'ImportDeclaration' && item.source.value === 'babel-plugin-number-calc-autorepair/dist/number-acc.js';
  })[0];

  if (item) {
    var existOne = item.specifiers.filter(function (item) {
      return item.imported.type === 'Identifier' && item.imported.name === key;
    })[0];

    if (!existOne) {
      var newOne = types.importSpecifier(types.identifier("_".concat(key)), types.identifier(key));
      item.specifiers.push(newOne);
      return newOne.local.name;
    }

    return existOne.local.name;
  } else {
    return (0, _helperModuleImports.addNamed)(nodePath, key, 'babel-plugin-number-calc-autorepair/dist/number-acc.js').name;
  }
}